__title__ = "mathy_envs"
__version__ = "0.12.1"
__summary__ = "Learning environments for solving math problems step-by-step"
__uri__ = "https://mathy.ai"
__author__ = "Justin DuJardin"
__email__ = "justin@dujardinconsulting.com"
__license__ = "All rights reserved"
